@@ -17,7 +17,7 @@ import json
 import logging
 import os.path
 import typing
-from typing import List, Optional
+from typing import List, Optional, Union
 from uuid import UUID
 
 import requests
@@ -30,28 +30,39 @@ import requests
 # are processed.
 #
 # This is supported in Python 3.7+
-
 if typing.TYPE_CHECKING:
     import pandas
     import pyspark.sql
     import s3fs
     from google.cloud import bigquery
 
-from .checks import Check
-from .cluster import Cluster
-from .destination import (
-    Destination, BigQueryDestination, GCSDestination, HDFSDestination, LocalDestination, S3ADestination, S3Destination
+from .models.branch import BranchName
+from .models.checks import Check, CheckCreationRequest, CheckId
+from .models.cluster import Cluster, ClusterId, ClusterName
+from .models.commit import Commit, CommitId
+from .models.destination import (
+    Destination, BigQueryDestination, GCSDestination, HDFSDestination, LocalDestination, S3ADestination,
+    S3Destination, DestinationId, DestinationName
 )
-from .feature import BareFeature, FeatureTemplate, GeneratedFeatures
-from .feature.run import FeatureRunSummary
-from .merge_request import MergeRequest
-from .model import (
-    FeatureStoreRun, FeatureSet, FeatureStore, Commit, Ref, TableDestinationReference, FolderDestinationReference,
-    FileFormat, RunStatus, Parquet, Orc, Csv
-)
-from .table import Table
+from .models.destination_reference import TableDestinationReference, FolderDestinationReference
+from .models.generated_features import GeneratedFeatures
+from .models.feature import Feature
+from .models.feature_creation_request import FeatureCreationRequest
+from .models.feature_id import FeatureId, FeatureName
+from .models.feature_run_summaries import FeatureRunSummary
+from .models.feature_set import FeatureSet, FeatureSetId, FeatureSetName
+from .models.feature_store import FeatureStore, FeatureStoreName, FeatureStoreId
+from .models.feature_store_run import FeatureStoreRun
+from .models.feature_template import FeatureTemplate, TemplateId, TemplateName
+from .models.feature_template_creation_request import FeatureTemplateCreationRequest
+from .models.file_format import CSV, FileFormat, Parquet, Orc
+from .models.jobs import RunStatus, FeatureStoreRunId
+from .models.merge_request import MergeRequest, MergeRequestId
+from .models.ref import Ref
+from .models.table import Table, TableId
 from . import version
 
+
 __version__ = version.__version__
 
 log = logging.getLogger(__name__)
@@ -85,7 +96,7 @@ class Anaml:
         self._token = base64.b64encode(bytes(apikey + ':' + secret, 'utf-8')).decode('utf-8')
         self._headers = {'Authorization': 'Basic ' + self._token}
         if ref is not None:
-            self._ref = {ref.ref_type: ref.ref}
+            self._ref = {ref.adt_type: ref.ref}
         else:
             self._ref = {}
 
@@ -161,18 +172,18 @@ class Anaml:
 
     # Commits and Branches
 
-    def get_current_commit(self, branch: str) -> Commit:
+    def get_current_commit(self, branch: Union[BranchName, str]) -> Commit:
         """Get the current commit for a branch.
 
         Args:
-            branch (str): Name of the branch to inspect.
+            branch: Name of the branch to inspect.
 
         Returns:
             The commit currently at the tip of the named branch.
         """
-        r = self._get(f"/branch/{branch}")
+        r = self._get(f"/branch/{str(branch)}")
         result = self._json_or_handle_errors(r)
-        return Commit.from_dict(result)
+        return Commit.from_json(result)
 
     # Cluster-related functions
 
@@ -184,36 +195,36 @@ class Anaml:
         """
         r = self._get("/cluster")
         result = self._json_or_handle_errors(r)
-        return [Cluster.from_dict(d) for d in result]
+        return [Cluster.from_json(d) for d in result]
 
-    def get_cluster_by_id(self, id: int) -> Cluster:
+    def get_cluster_by_id(self, cluster_id: Union[ClusterId, int]) -> Cluster:
         """Get a cluster definition from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the cluster definition to retrieve.
+            cluster_id: Unique identifier of the cluster definition to retrieve.
 
         Returns:
             The requested cluster definition, if it exists.
         """
-        r = self._get(f"/cluster/{id}")
+        r = self._get(f"/cluster/{int(cluster_id)}")
         result = self._json_or_handle_errors(r)
-        return Cluster.from_dict(result)
+        return Cluster.from_json(result)
 
-    def get_cluster_by_name(self, name: str) -> Cluster:
+    def get_cluster_by_name(self, name: Union[ClusterName, str]) -> Cluster:
         """Get a cluster definition from the Anaml server.
 
         Args:
-            name (str): Name of cluster definition to retrieve.
+            name: Name of cluster definition to retrieve.
 
         Returns:
             The requested cluster definition, if it exists.
         """
-        r = self._get("/cluster", query={'name': name})
+        r = self._get("/cluster", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return Cluster.from_dict(result)
+        return Cluster.from_json(result)
 
     # Feature-related functions
-    def get_features(self) -> List[BareFeature]:
+    def get_features(self) -> List[Feature]:
         """Get a list of all features from the Anaml server.
 
         Returns:
@@ -221,76 +232,44 @@ class Anaml:
         """
         r = self._get("/feature")
         result = self._json_or_handle_errors(r)
-        return [BareFeature.from_json(d) for d in result]
+        return [Feature.from_json(d) for d in result]
 
-    def get_feature_templates(self) -> List[FeatureTemplate]:
-        """Get a list of all features templates from the Anaml server.
-
-        Returns:
-            A list of all feature templates.
-        """
-        r = self._get("/feature-template")
-        result = self._json_or_handle_errors(r)
-        return [FeatureTemplate.from_json(d) for d in result]
-
-    def get_feature_by_id(self, id: int) -> BareFeature:
+    def get_feature_by_id(self, feature_id: Union[FeatureId, int]) -> Feature:
         """Get a feature from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the feature to retrieve.
+            feature_id: Unique identifier of the feature to retrieve.
 
         Returns:
             The requested feature, if it exists.
         """
-        r = self._get("/feature/" + str(id))
+        r = self._get(f"/feature/{int(feature_id)}")
         result = self._json_or_handle_errors(r)
-        return BareFeature.from_json(result)
+        return Feature.from_json(result)
 
-    def get_feature_by_name(self, name: str) -> BareFeature:
+    def get_feature_by_name(self, name: Union[FeatureName, str]) -> Feature:
         """Get a feature from the Anaml server.
 
         Args:
-            name (str): Name of the feature to retrieve.
+            name: Name of the feature to retrieve.
 
         Returns:
             The requested feature, if it exists.
         """
-        r = self._get("/feature", query={'name': name})
+        r = self._get("/feature", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return BareFeature.from_json(result)
+        return Feature.from_json(result)
 
-    def get_feature_template_by_id(self, id: int) -> FeatureTemplate:
-        """Get a feature template from the Anaml server.
-
-        Args:
-            id (int): Unique identifier of the feature template to retrieve.
-
-        Returns:
-            The requested feature template, if it exists.
-        """
-        r = self._get("/feature-template/" + str(id))
-        result = self._json_or_handle_errors(r)
-        return FeatureTemplate.from_json(result)
-
-    def get_feature_template_by_name(self, name: str) -> FeatureTemplate:
-        """Get a feature template from the Anaml server.
-
-        Args:
-            name (str): Name of the feature template to retrieve.
-
-        Returns:
-            The requested feature template, if it exists.
-        """
-        r = self._get("/feature-template", query={'name': name})
-        result = self._json_or_handle_errors(r)
-        return FeatureTemplate.from_json(result)
-
-    def get_generated_features(self, feature_store: str, primary_key: int) -> GeneratedFeatures:
+    def get_generated_features(
+        self,
+        feature_store: Union[FeatureStoreName, str],
+        primary_key: int
+    ) -> GeneratedFeatures:
         """Get the features generated from a feature store for a particular primary key value.
 
         Args:
-            feature_store (str): Name of the feature store.
-            primary_key (int): Primary key of the entity to get.
+            feature_store: Name of the feature store.
+            primary_key: Primary key of the entity to get.
 
         Returns:
             The features for the given primary key in the named feature store.
@@ -299,31 +278,36 @@ class Anaml:
         result = self._json_or_handle_errors(r)
         return GeneratedFeatures.from_json(result)
 
-    def create_feature(self, feature: BareFeature) -> BareFeature:
-        """Create or update a feature definition on the Anaml server.
+    def create_feature(self, feature: FeatureCreationRequest) -> Feature:
+        """Create a feature definition on the Anaml server.
 
         Args:
             feature: The feature definition.
 
-        If the feature object contains an ID, that feature will be updated;
-        otherwise a new feature will be created.
+        Returns:
+            The new feature.
+        """
+        r = self._post("/feature", json=feature.to_json())
+        id = self._int_or_handle_errors(r)
+        return self.get_feature_by_id(id)
+
+    def update_feature(self, feature: Feature) -> Feature:
+        """Update an existing feature definition on the Anaml server.
+
+        Args:
+            feature: The feature definition.
 
         Returns:
             The feature definition, with its unique identifier and other computed
             fields updated.
         """
-        endpoint = "/feature-template" if feature.is_template() else "/feature"
-        if feature.id is not None:
-            r = self._put(endpoint + "/" + str(feature.id), json=feature.to_json())
-            id = self._int_or_handle_errors(r)
-            return feature.copy(id=id)
-        else:
-            r = self._post(endpoint, json=feature.to_json())
-            id = self._int_or_handle_errors(r)
-            return feature.copy(id=id)
+        r = self._put(f"/feature/{str(feature.id.value)}", json=feature.to_json())
+        # TODO: Is this, in fact, an int?
+        _ = self._int_or_handle_errors(r)
+        return self.get_feature_by_id(int(feature.id.value))
 
-    def preview_feature(self, feature: BareFeature) -> None:
-        """Show a matplotlib plot for the preview statistics of a feature.
+    def preview_feature(self, feature: Feature) -> List:
+        """Returns list of feature statistics.
 
         Args:
             feature: a Feature object
@@ -331,12 +315,20 @@ class Anaml:
         r = self._post("/feature-preview", json={"features": [feature.to_json()]})
         result = self._json_or_handle_errors(r)
 
-        feature_stats = [
+        return [
             fs
             for pd in self._to_list(result.get("previewData"))
             for fss in self._to_list(pd.get("featureStatistics"))
             for fs in fss
         ]
+
+    def plot_feature_preview(self, feature: Feature) -> None:
+        """Show a matplotlib plot for the preview statistics of a feature.
+
+        Args:
+            feature: a Feature object
+        """
+        feature_stats = self.preview_feature(feature)
         for fs in feature_stats:
             self._build_feature_plots(fs)
         self._warn_empty_feature_stats([
@@ -344,7 +336,7 @@ class Anaml:
             for fs in feature_stats if fs.get("adt_type") == "empty"
         ])
 
-    def sample_feature(self, feature: BareFeature) -> pandas.DataFrame:
+    def sample_feature(self, feature: Feature) -> pandas.DataFrame:
         """Generate a sample of feature values.
 
         Arguments:
@@ -360,6 +352,55 @@ class Anaml:
 
         return pandas.DataFrame(result)
 
+    def create_feature_template(self, template: FeatureTemplateCreationRequest) -> FeatureTemplate:
+        """Create a new feature template.
+
+        Args:
+            template: Details of the feature template.
+
+        Returns:
+            The new feature template.
+        """
+        r = self._post("/feature-template", json=template.to_json())
+        id = self._int_or_handle_errors(r)
+        return self.get_feature_template_by_id(id)
+
+    def get_feature_templates(self) -> List[FeatureTemplate]:
+        """Get a list of all features templates from the Anaml server.
+
+        Returns:
+            A list of all feature templates.
+        """
+        r = self._get("/feature-template")
+        result = self._json_or_handle_errors(r)
+        return [FeatureTemplate.from_json(d) for d in result]
+
+    def get_feature_template_by_id(self, feature_template_id: Union[TemplateId, int]) -> FeatureTemplate:
+        """Get a feature template from the Anaml server.
+
+        Args:
+            feature_template_id: Unique identifier of the feature template to retrieve.
+
+        Returns:
+            The requested feature template, if it exists.
+        """
+        r = self._get(f"/feature-template/{int(feature_template_id)}")
+        result = self._json_or_handle_errors(r)
+        return FeatureTemplate.from_json(result)
+
+    def get_feature_template_by_name(self, name: Union[TemplateName, str]) -> FeatureTemplate:
+        """Get a feature template from the Anaml server.
+
+        Args:
+            name: Name of the feature template to retrieve.
+
+        Returns:
+            The requested feature template, if it exists.
+        """
+        r = self._get("/feature-template", query={'name': str(name)})
+        result = self._json_or_handle_errors(r)
+        return FeatureTemplate.from_json(result)
+
     # Table-related functions
     def get_tables(self) -> List[Table]:
         """Get a list of all tables from the Anaml server.
@@ -369,33 +410,33 @@ class Anaml:
         """
         r = self._get("/table")
         result = self._json_or_handle_errors(r)
-        return [Table.from_dict(d) for d in result]
+        return [Table.from_json(d) for d in result]
 
-    def get_table_by_id(self, id: int) -> Table:
+    def get_table_by_id(self, table_id: Union[TableId, int]) -> Table:
         """Get a table from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the table to retrieve.
+            table_id: Unique identifier of the table to retrieve.
 
         Returns:
             The requested table, if it exists.
         """
-        r = self._get("/table/" + str(id))
+        r = self._get(f"/table/{int(table_id)}")
         result = self._json_or_handle_errors(r)
-        return Table.from_dict(result)
+        return Table.from_json(result)
 
     def get_table_by_name(self, name: str) -> Table:
         """Get a table from the Anaml server.
 
         Args:
-            name (str): Name of the table to retrieve.
+            name: Name of the table to retrieve.
 
         Returns:
             The requested table, if it exists.
         """
-        r = self._get("/table", query={'name': name})
+        r = self._get("/table", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return Table.from_dict(result)
+        return Table.from_json(result)
 
     # Destination-related functions
     def get_destinations(self) -> List[Destination]:
@@ -406,44 +447,44 @@ class Anaml:
         """
         r = self._get("/destination")
         result = self._json_or_handle_errors(r)
-        return [Destination.from_dict(d) for d in result]
+        return [Destination.from_json(d) for d in result]
 
-    def get_destination_by_id(self, id: int) -> Destination:
+    def get_destination_by_id(self, destination_id: Union[DestinationId, int]) -> Destination:
         """Get a destination from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the destination.
+            destination_id: Unique identifier of the destination.
 
         Returns:
             The destination, if it exists.
         """
-        r = self._get(f"/destination/{id}")
+        r = self._get(f"/destination/{int(destination_id)}")
         result = self._json_or_handle_errors(r)
-        return Destination.from_dict(result)
+        return Destination.from_json(result)
 
-    def get_destination_by_name(self, name: str) -> Destination:
+    def get_destination_by_name(self, name: Union[DestinationName, str]) -> Destination:
         """Get a destination from the Anaml server.
 
         Args:
-            name (str): Name of the destination.
+            name: Name of the destination.
 
         Returns:
             The destination, if it exists.
         """
-        r = self._get("/destination", query={'name': name})
+        r = self._get("/destination", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return Destination.from_dict(result)
+        return Destination.from_json(result)
 
-    def get_feature_run_summary(self, feature_id: int) -> FeatureRunSummary:
+    def get_feature_run_summary(self, feature_id: Union[FeatureId, int]) -> FeatureRunSummary:
         """Get a summary of the most recent run of a feature.
 
         Args:
-            feature_id (int): Unique identifier of the feature.
+            feature_id: Unique identifier of the feature.
 
         Returns:
             A summary of the given feature from the most recent feature store run.
         """
-        r = self._get(f"/feature/{feature_id}/latest-run-statistics")
+        r = self._get(f"/feature/{int(feature_id)}/latest-run-statistics")
         result = self._json_or_handle_errors(r)
         return FeatureRunSummary.from_json(result)
 
@@ -455,46 +496,46 @@ class Anaml:
         """
         r = self._get("/feature-set")
         result = self._json_or_handle_errors(r)
-        return [FeatureSet.from_dict(r) for r in result]
+        return [FeatureSet.from_json(r) for r in result]
 
-    def get_feature_set_by_id(self, id: int) -> FeatureSet:
+    def get_feature_set_by_id(self, feature_set_id: Union[FeatureSetId, int]) -> FeatureSet:
         """Get a feature set from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the feature set.
+            feature_set_id: Unique identifier of the feature set.
 
         Returns:
             The feature set, if it exists.
         """
-        r = self._get(f"/feature-set/{id}")
+        r = self._get(f"/feature-set/{int(feature_set_id)}")
         result = self._json_or_handle_errors(r)
-        return FeatureSet.from_dict(result)
+        return FeatureSet.from_json(result)
 
-    def get_feature_set_by_name(self, name: str) -> FeatureSet:
+    def get_feature_set_by_name(self, name: Union[FeatureSetName, str]) -> FeatureSet:
         """Get a feature set from the Anaml server.
 
         Args:
-            name (str): Name of the feature set.
+            name: Name of the feature set.
 
         Returns:
             The feature set, if it exists.
         """
-        r = self._get("/feature-set", query={'name': name})
+        r = self._get("/feature-set", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return FeatureSet.from_dict(result)
+        return FeatureSet.from_json(result)
 
-    def get_run_for_feature_set(self, feature_set_id: int) -> FeatureStoreRun:
+    def get_run_for_feature_set(self, feature_set_id: Union[FeatureSetId, int]) -> FeatureStoreRun:
         """Get the most recent feature store run for the given feature set.
 
         Args:
-            feature_set_id (int): The unique identifier of the feature set.
+            feature_set_id: The unique identifier of the feature set.
 
         Returns:
             The most recent feature store run for that feature set.
         """
-        r = self._get(f"/feature-set/{feature_set_id}/latest-run-statistics")
+        r = self._get(f"/feature-set/{int(feature_set_id)}/latest-run-statistics")
         result = self._json_or_handle_errors(r)
-        return FeatureStoreRun.from_dict(result)
+        return FeatureStoreRun.from_json(result)
 
     def get_feature_stores(self) -> List[FeatureStore]:
         """Get a list of all feature stores from the Anaml server.
@@ -504,68 +545,95 @@ class Anaml:
         """
         r = self._get("/feature-store")
         result = self._json_or_handle_errors(r)
-        return [
-            FeatureStore.from_dict(r) for r in result
-        ]
+        return [FeatureStore.from_json(r) for r in result]
 
-    def get_feature_store_by_id(self, id: int) -> FeatureStore:
+    def get_feature_store_by_id(self, feature_store_id: Union[FeatureStoreId, int]) -> FeatureStore:
         """Get a feature store from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the feature store.
+            feature_store_id: Unique identifier of the feature store.
 
         Returns:
             The feature store, if it exists.
         """
-        r = self._get(f"/feature-store/{id}")
+        r = self._get(f"/feature-store/{int(feature_store_id)}")
         result = self._json_or_handle_errors(r)
-        return FeatureStore.from_dict(result)
+        return FeatureStore.from_json(result)
 
-    def get_feature_store_by_name(self, name: str) -> FeatureStore:
+    def get_feature_store_by_name(self, name: Union[FeatureStoreName, str]) -> FeatureStore:
         """Get a feature store from the Anaml server.
 
         Args:
-            name (str): Name of the feature store.
+            name: Name of the feature store.
 
         Returns:
             The feature store, if it exists.
         """
-        r = self._get("/feature-store", query={'name': name})
+        r = self._get("/feature-store", query={'name': str(name)})
         result = self._json_or_handle_errors(r)
-        return FeatureStore.from_dict(result)
+        return FeatureStore.from_json(result)
 
-    def get_feature_store_runs(self, feature_store_id: int) -> List[FeatureStoreRun]:
+    def get_feature_store_runs(
+        self,
+        feature_store_id: Union[FeatureStoreId, int],
+        num: Optional[int] = None
+    ) -> List[FeatureStoreRun]:
         """Get a list of all runs of a given feature store from the Anaml server.
 
         Args:
             feature_store_id: The unique identifier of a feature store.
+            num: Optional. Maximum number of results to return.
 
         Returns:
             A list of runs of the given feature store.
         """
-        r = self._get(f"/feature-store/{feature_store_id}/run")
+        q = {}
+        if num is not None:
+            q['num'] = num
+        r = self._get(f"/feature-store/{int(feature_store_id)}/run", query=q)
         result = self._json_or_handle_errors(r)
-        return [FeatureStoreRun.from_dict(r) for r in result]
+        return [FeatureStoreRun.from_json(r) for r in result]
 
-    def get_feature_store_run(self, feature_store_id: int, run_id: int) -> FeatureStoreRun:
+    def get_feature_store_run(
+        self,
+        feature_store_id: Union[FeatureStoreId, int],
+        run_id: Union[FeatureStoreRunId, int]
+    ) -> FeatureStoreRun:
         """Get the details for a feature store run from the Anaml server.
 
         Args:
-            feature_store_id (int): The unique identifier of a feature store.
-            run_id (int): The unique identifier of a run of that feature store.
+            feature_store_id: The unique identifier of a feature store.
+            run_id: The unique identifier of a run of that feature store.
 
         Returns:
              Details of the given feature store run.
         """
-        r = self._get(f"/feature-store/{feature_store_id}/run/{run_id}")
+        r = self._get(f"/feature-store/{int(feature_store_id)}/run/{int(run_id)}")
         result = self._json_or_handle_errors(r)
-        return FeatureStoreRun.from_dict(result)
+        return FeatureStoreRun.from_json(result)
+
+    def get_latest_feature_store_run_by_name(self, feature_store_name: Union[FeatureStoreName, str]) -> FeatureStoreRun:
+        """Get the most recent run of the named feature store from the Anaml server.
+
+        Args:
+            feature_store_name: The name of the feature store.
+
+        Returns:
+            Details of the feature store run.
+
+        Raises:
+            IndexError: When the named feature store has no runs.
+        """
+        feature_store = self.get_feature_store_by_name(feature_store_name)
+        # TODO: Extend server to provide an end
+        runs = self.get_feature_store_runs(feature_store_id=feature_store.id.value, num=1)
+        return runs[0]
 
     def get_merge_requests(self, limit: int = 10) -> List[MergeRequest]:
         """List merge requests from the Anaml server.
 
         Args:
-            limit (int): The maximum number of objects to return.
+            limit: The maximum number of objects to return.
 
         Returns:
             A list of merge requests.
@@ -573,53 +641,67 @@ class Anaml:
         r = self._get("/merge-request")
         result = self._json_or_handle_errors(r)
         return [
-            MergeRequest.from_dict(r) for r in result
+            MergeRequest.from_json(r) for r in result
         ]
 
-    def get_merge_request(self, id: int) -> MergeRequest:
+    def get_merge_request(self, merge_request_id: Union[MergeRequestId, int]) -> MergeRequest:
         """Get a merge request from the Anaml server.
 
         Args:
-            id (int): Unique identifier of the merge request.
+            merge_request_id: Unique identifier of the merge request.
 
         Returns:
             The merge request, if it exists.
         """
-        r = self._get(f"/merge-request/{id}")
+        r = self._get(f"/merge-request/{int(merge_request_id)}")
         result = self._json_or_handle_errors(r)
-        return MergeRequest.from_dict(result)
+        return MergeRequest.from_json(result)
 
     # Checks
-    def get_checks(self, commit_id: UUID) -> List[Check]:
+    def get_checks(self, commit_id: Union[CommitId, UUID]) -> List[Check]:
         """Get checks for a given commit from the Anaml server.
 
         Args:
-            commit_id (UUID): Unique identifier of the commit.
+            commit_id: Unique identifier of the commit.
 
         Returns:
             A list of checks associated with the given commit.
         """
-        r = self._get(f"/checks/{commit_id}")
+        r = self._get(f"/checks/{str(commit_id)}")
         result = self._json_or_handle_errors(r)
         return [
-            Check.from_dict(r) for r in result
+            Check.from_json(r) for r in result
         ]
 
-    def get_check(self, commit_id: UUID, check_id: int) -> Check:
+    def get_check(self, commit_id: Union[CommitId, UUID], check_id: Union[CheckId, int]) -> Check:
         """Get a specific check for given a commit_id from the Anaml server.
 
         Args:
-            commit_id (UUID): Unique identifier of the commit.
-            check_id (int): Unique identifier of the check.
+            commit_id: Unique identifier of the commit.
+            check_id: Unique identifier of the check.
 
         Returns:
             The check, if it exists.
         """
-        r = self._get(f"/checks/{commit_id}/{check_id}")
+        r = self._get(f"/checks/{str(commit_id)}/{int(check_id)}")
         result = self._json_or_handle_errors(r)
-        return Check.from_dict(result)
+        return Check.from_json(result)
 
-    def save_check(self, check: Check) -> Check:
+    def create_check(self, commit: Union[CommitId, UUID], check: CheckCreationRequest) -> Check:
+        """Create a new Check for a commit.
+
+        Args:
+            commit: The commit being checked.
+            check: The details of the check.
+
+        Returns:
+            The check object, with unique identifier and other computed fields updated.
+        """
+        r = self._post(f"/checks/{str(commit)}", json=check.to_json())
+        id = self._int_or_handle_errors(r)
+        return self.get_check(commit, id)
+
+    def update_check(self, check: Check) -> Check:
         """Get Checks from the Anaml server.
 
         Args:
@@ -628,14 +710,9 @@ class Anaml:
         Returns:
             The check object, with unique identifier and other computed fields updated.
         """
-        if check.id is not None:
-            self._put("/checks/" + str(check.commit) + "/" + str(check.id), json=check.to_json())
-        else:
-            result = self._post("/checks/" + str(check.commit), json=check.to_json())
-            check_dict = check.to_dict()
-            check_dict['id'] = int(result.content)
-            check = Check(**check_dict)
-        return check
+        r = self._put("/checks/" + str(check.commit.value) + "/" + str(check.id.value), json=check.to_json())
+        result = self._json_or_handle_errors(r)
+        return Check.from_json(result)
 
     #####################
     # Load feature data #
@@ -689,11 +766,11 @@ class Anaml:
             run: FeatureStoreRun,
             *,
             spark_session: Optional[pyspark.sql.SparkSession] = None,
-    ) -> typing.Union[pandas.DataFrame, pyspark.sql.DataFrame]:
+    ) -> Union[pandas.DataFrame, pyspark.sql.DataFrame]:
         """Load the data from a feature store into a data frame.
 
         Args:
-            run (FeatureStoreRun): A run from a feature store.
+            run: A run from a feature store.
             spark_session: Optional Spark session to load the data.
 
         Returns:
@@ -701,15 +778,15 @@ class Anaml:
               Otherwise, a Pandas data frame will be created and returned.
         """
         if run.status != RunStatus.Completed:
-            log.debug(f"Attempted to load data from feature store run id={run.id}, status={run.status.value}")
+            log.debug(f"Attempted to load data from feature store run id={run.id.value}, status={run.status.value}")
             raise ValueError("The feature store run is not complete")
         # TODO: We should thing about using Version here.
-        store = self.get_feature_store_by_id(id=run.featureStoreId)
+        store = self.get_feature_store_by_id(run.featureStoreId.value)
         # Loop through the destinations and attempt to load them. They should all contain the same data, so we'll
         # take the first one we find that actually returns a dataframe.
         dataframe = None
         for dest_ref in store.destinations:
-            dest = self.get_destination_by_id(id=dest_ref.destinationId)
+            dest = self.get_destination_by_id(dest_ref.destinationId.value)
             if isinstance(dest_ref, TableDestinationReference):
                 if isinstance(dest, BigQueryDestination) and spark_session is not None:
                     project, dataset = dest.path.split(":")
@@ -744,14 +821,14 @@ class Anaml:
                         prefix=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}"
                     )
                     spark_options = {}
-                    if isinstance(dest.fileFormat, Csv):
+                    if isinstance(dest.fileFormat, CSV):
                         spark_options['header'] = dest.fileFormat.includeHeader
                     dataframe = spark_session.read.load(path=url, format=dest.fileFormat.adt_type, **spark_options)
                 elif isinstance(dest, GCSDestination) and spark_session is None:
                     url = "gs://{bucket}/{prefix}/**{suffix}".format(
                         bucket=dest.bucket,
                         prefix=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}",
-                        suffix=dest.fileFormat.suffix
+                        suffix=self._file_format_suffix(dest.fileFormat)
                     )
                     dataframe = self._load_pandas_from_files(
                         urls=[url],
@@ -762,20 +839,20 @@ class Anaml:
                         path=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}"
                     )
                     spark_options = {}
-                    if isinstance(dest.fileFormat, Csv):
+                    if isinstance(dest.fileFormat, CSV):
                         spark_options['header'] = dest.fileFormat.includeHeader
                     dataframe = spark_session.read.load(path=url, format=dest.fileFormat.adt_type, **spark_options)
                 # TODO: Load Pandas data frame from HDFS.
                 elif isinstance(dest, LocalDestination) and spark_session is not None:
                     url = f"/{dest.path.strip('/')}/{dest_ref.folder.strip('/')}"
                     spark_options = {}
-                    if isinstance(dest.fileFormat, Csv):
+                    if isinstance(dest.fileFormat, CSV):
                         spark_options['header'] = dest.fileFormat.includeHeader
                     dataframe = spark_session.read.load(path=url, format=dest.fileFormat.adt_type, **spark_options)
                 elif isinstance(dest, LocalDestination) and spark_session is None:
                     url = "{prefix}/**/*{suffix}".format(
                         prefix=f"/{dest.path.strip('/')}/{dest_ref.folder.strip('/')}",
-                        suffix=dest.fileFormat.suffix
+                        suffix=self._file_format_suffix(dest.fileFormat)
                     )
                     dataframe = self._load_pandas_from_files(
                         urls=filter(os.path.isfile, glob.iglob(url, recursive=True)),
@@ -787,7 +864,7 @@ class Anaml:
                         prefix=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}",
                     )
                     spark_options = {}
-                    if isinstance(dest.fileFormat, Csv):
+                    if isinstance(dest.fileFormat, CSV):
                         spark_options['header'] = dest.fileFormat.includeHeader
                     dataframe = spark_session.read.load(path=url, format=dest.fileFormat.adt_type, **spark_options)
                 elif isinstance(dest, S3Destination) and spark_session is not None:
@@ -796,14 +873,14 @@ class Anaml:
                         prefix=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}"
                     )
                     spark_options = {}
-                    if isinstance(dest.fileFormat, Csv):
+                    if isinstance(dest.fileFormat, CSV):
                         spark_options['header'] = dest.fileFormat.includeHeader
                     dataframe = spark_session.read.load(path=url, format=dest.fileFormat.adt_type, **spark_options)
                 elif isinstance(dest, S3Destination) or isinstance(dest, S3ADestination):
                     url = "{bucket}/{prefix}/**{suffix}".format(
                         bucket=dest.bucket,
                         prefix=f"{dest.path.strip('/')}/{dest_ref.folder.strip('/')}",
-                        suffix=dest.fileFormat.suffix
+                        suffix=self._file_format_suffix(dest.fileFormat)
                     )
                     dataframe = self._load_pandas_from_files(
                         urls=self._s3fs_client.glob(path=url),
@@ -827,8 +904,8 @@ class Anaml:
         """Load a folder of datafiles in Google Cloud Storage into a Pandas data frame.
 
         Args:
-            urls (Iterable[str]): Collection of paths/URLs to the data files.
-            format (FileFormat): Format of the data files.
+            urls: Collection of paths/URLs to the data files.
+            format: Format of the data files.
 
         Warning: This method makes no attempt to check that the requested data will fit into available memory.
         """
@@ -837,7 +914,7 @@ class Anaml:
             return pandas.concat(pandas.read_parquet(url) for url in urls)
         elif isinstance(format, Orc):
             return pandas.concat(pandas.read_orc(url) for url in urls)
-        elif isinstance(format, Csv):
+        elif isinstance(format, CSV):
             return pandas.concat(pandas.read_csv(url) for url in urls)
         else:
             raise ValueError(f"Cannot load unsupported format: {format.adt_type}")
@@ -913,3 +990,14 @@ class Anaml:
     def _warn_empty_feature_stats(features: List[str]):
         if features:
             log.warning(_NO_FEATURES.format(thefeatures=', '.join(features)))
+
+    @staticmethod
+    def _file_format_suffix(fmt: FileFormat):
+        if isinstance(fmt, Parquet):
+            return ".parquet"
+        elif isinstance(fmt, Orc):
+            return ".orc"
+        elif isinstance(fmt, CSV):
+            return ".csv"
+        else:
+            raise ValueError("Unknown file format: '{f}'".format(f=type(fmt).__name__))
